@@ -38,7 +38,7 @@ from plainbox.abc import IJobRunnerUI
 from plainbox.i18n import gettext as _
 from plainbox.i18n import ngettext
 from plainbox.i18n import pgettext as C_
-from plainbox.impl.color import ansi_on, ansi_off
+from plainbox.impl.color import Colorizer
 from plainbox.impl.commands.inv_checkbox import CheckBoxInvocationMixIn
 from plainbox.impl.depmgr import DependencyDuplicateError
 from plainbox.impl.exporter import ByteStringStreamTranslator
@@ -58,69 +58,6 @@ from plainbox.impl.transport import TransportError
 logger = logging.getLogger("plainbox.commands.run")
 
 
-class Colorizer:
-    """
-    Colorizing helper for various kinds of content we need to handle
-    """
-
-    # NOTE: Ideally result and all would be handled by multi-dispatch __call__
-
-    def __init__(self, color):
-        self.c = color
-
-    def result(self, result):
-        outcome_color = {
-            IJobResult.OUTCOME_PASS: "GREEN",
-            IJobResult.OUTCOME_FAIL: "RED",
-            IJobResult.OUTCOME_SKIP: "YELLOW",
-            IJobResult.OUTCOME_UNDECIDED: "MAGENTA",
-            IJobResult.OUTCOME_NOT_SUPPORTED: "YELLOW",
-        }.get(result.outcome, "RESET")
-        return self(result.tr_outcome(), outcome_color)
-
-    def header(self, text, color_name='WHITE', bright=True, fill='='):
-        return self("[ {} ]".format(text).center(80, fill), color_name, bright)
-
-    def f(self, color_name):
-        return getattr(self.c.f, color_name.upper())
-
-    def b(self, color_name):
-        return getattr(self.c.b, color_name.upper())
-
-    def s(self, style_name):
-        return getattr(self.c.s, style_name.upper())
-
-    def __call__(self, text, color_name="WHITE", bright=True):
-        return ''.join([
-            self.f(color_name),
-            self.c.s.BRIGHT if bright else '', str(text),
-            self.c.s.RESET_ALL])
-
-    def BLACK(self, text, bright=True):
-        return self(text, "BLACK", bright)
-
-    def RED(self, text, bright=True):
-        return self(text, "RED", bright)
-
-    def GREEN(self, text, bright=True):
-        return self(text, "GREEN", bright)
-
-    def YELLOW(self, text, bright=True):
-        return self(text, "YELLOW", bright)
-
-    def BLUE(self, text, bright=True):
-        return self(text, "BLUE", bright)
-
-    def MAGENTA(self, text, bright=True):
-        return self(text, "MAGENTA", bright)
-
-    def CYAN(self, text, bright=True):
-        return self(text, "CYAN", bright)
-
-    def WHITE(self, text, bright=True):
-        return self(text, "WHITE", bright)
-
-
 Action = collections.namedtuple("Action", "accel label cmd")
 
 
@@ -142,7 +79,7 @@ class ActionUI:
             prompt = _("Pick an action")
         self.action_list = action_list
         self.prompt = prompt
-        self.C = Colorizer(color or ansi_off)
+        self.C = Colorizer(color)
 
     def run(self):
         long_hint = "\n".join(
@@ -193,6 +130,15 @@ class SilentUI(IJobRunnerUI):
     def notify_about_description(self, job):
         pass
 
+    def notify_about_purpose(self, job):
+        pass
+
+    def notify_about_steps(self, job):
+        pass
+
+    def notify_about_verification(self, job):
+        pass
+
     def job_cannot_start(self, job, job_state, job_result):
         pass
 
@@ -218,7 +164,7 @@ class NormalUI(IJobRunnerUI):
     def considering_job(self, job, job_state):
         print(self.C.header(job.tr_summary(), fill='-'))
         print(_("ID: {0}").format(job.id))
-        print(_("Category: {0}").format(job.category_id))
+        print(_("Category: {0}").format(job_state.effective_category_id))
 
     def about_to_start_running(self, job, job_state):
         pass
@@ -259,7 +205,22 @@ class NormalUI(IJobRunnerUI):
         pass
 
     def notify_about_description(self, job):
-        print(self.C.CYAN(job.tr_description()))
+        if job.tr_description() is not None:
+            print(self.C.CYAN(job.tr_description()))
+
+    def notify_about_purpose(self, job):
+        if job.tr_purpose() is not None:
+            print(self.C.CYAN(job.tr_purpose()))
+        else:
+            self.notify_about_description(job)
+
+    def notify_about_steps(self, job):
+        if job.tr_steps() is not None:
+            print(self.C.CYAN(job.tr_steps()))
+
+    def notify_about_verification(self, job):
+        if job.tr_verification() is not None:
+            print(self.C.CYAN(job.tr_verification()))
 
     def job_cannot_start(self, job, job_state, result):
         print(_("Job cannot be started because:"))
@@ -303,16 +264,17 @@ class RunInvocation(CheckBoxInvocationMixIn):
         time the loop-over-all-jobs is started.
     """
 
-    def __init__(self, provider_loader, config, ns, use_colors=True):
-        super().__init__(provider_loader, config)
+    def __init__(self, provider_loader, config_loader, ns, color):
+        super().__init__(provider_loader, config_loader)
         self.ns = ns
         self._manager = None
         self._runner = None
         self._exporter = None
         self._transport = None
         self._backtrack_and_run_missing = True
-        self._color = ansi_on if use_colors else ansi_off
-        self.C = Colorizer(self._color)
+        self._color = color
+        self._test_plan = self.find_test_plan()
+        self.C = Colorizer(color)
 
     @property
     def manager(self):
@@ -401,6 +363,8 @@ class RunInvocation(CheckBoxInvocationMixIn):
             resumed = False
         # Create the job runner so that we can do stuff
         self.create_runner()
+        # Set the effective category for each job
+        self.set_effective_categories()
         # If we haven't resumed then do some one-time initialization
         if not resumed:
             # Store the application-identifying meta-data and checkpoint the
@@ -580,7 +544,11 @@ class RunInvocation(CheckBoxInvocationMixIn):
         else:
             option_list = None
         try:
-            self._exporter = exporter_cls(option_list)
+            if self.ns.output_format == 'text':
+                # Hacky way to pass color flag down to the text exporter
+                self._exporter = exporter_cls(option_list, self.ns.color)
+            else:
+                self._exporter = exporter_cls(option_list)
         except ValueError as exc:
             raise SystemExit(str(exc))
 
@@ -679,6 +647,26 @@ class RunInvocation(CheckBoxInvocationMixIn):
     def expected_session_title(self):
         return " ".join([os.path.basename(sys.argv[0])] + sys.argv[1:])
 
+    def find_test_plan(self):
+        # This is using getattr because the code is shared with checkbox-ng
+        # that doesn't support the same set of command line options.
+        test_plan_id = getattr(self.ns, "test_plan", None)
+        if test_plan_id is None:
+            return
+        for provider in self.provider_list:
+            for unit in provider.get_units()[0]:
+                if (unit.Meta.name == 'test plan'
+                        and unit.id == test_plan_id):
+                    return unit
+
+    def set_effective_categories(self):
+        if self._test_plan is None:
+            return
+        ecm = self._test_plan.get_effective_category_map(self.state.job_list)
+        for job_id, effective_category_id in ecm.items():
+            job_state = self.state.job_state_map[job_id]
+            job_state.effective_category_id = effective_category_id
+
     def do_initial_job_selection(self):
         """
         Compute the initial list of desired jobs
@@ -757,28 +745,29 @@ class RunInvocation(CheckBoxInvocationMixIn):
         while True:
             if job.plugin in ('user-interact', 'user-interact-verify',
                               'user-verify', 'manual'):
-                ui.notify_about_description(job)
-            if (self.is_interactive and
-                    job.plugin in ('user-interact', 'user-interact-verify')):
-                cmd = ui.wait_for_interaction_prompt(job)
-                if cmd == 'run' or cmd is None:
-                    job_result = self.runner.run_job(job, self.config, ui)
-                elif cmd == 'comment':
-                    new_comment = input(self.C.BLUE(
-                        _('Please enter your comments:') + '\n'))
-                    if new_comment:
-                        comments += new_comment + '\n'
-                    continue
-                elif cmd == 'skip':
-                    job_result = MemoryJobResult({
-                        'outcome': IJobResult.OUTCOME_SKIP,
-                        'comments': _("Explicitly skipped before execution")
-                    })
-                    if comments != "":
-                        job_result.comments = comments
-                    break
-                elif cmd == 'quit':
-                    raise SystemExit()
+                ui.notify_about_purpose(job)
+                if self.is_interactive:
+                    cmd = ui.wait_for_interaction_prompt(job)
+                    if cmd == 'run' or cmd is None:
+                        ui.notify_about_steps(job)
+                        job_result = self.runner.run_job(job, self.config, ui)
+                    elif cmd == 'comment':
+                        new_comment = input(self.C.BLUE(
+                            _('Please enter your comments:') + '\n'))
+                        if new_comment:
+                            comments += new_comment + '\n'
+                        continue
+                    elif cmd == 'skip':
+                        job_result = MemoryJobResult({
+                            'outcome': IJobResult.OUTCOME_SKIP,
+                            'comments': _("Explicitly skipped before"
+                                          " execution")
+                        })
+                        if comments != "":
+                            job_result.comments = comments
+                        break
+                    elif cmd == 'quit':
+                        raise SystemExit()
             else:
                 job_result = self.runner.run_job(job, self.config, ui)
             if (self.is_interactive and
@@ -786,6 +775,7 @@ class RunInvocation(CheckBoxInvocationMixIn):
                 try:
                     if comments != "":
                         job_result.comments = comments
+                    ui.notify_about_verification(job)
                     job_result = self._interaction_callback(
                         self.runner, job, job_result, self.config)
                 except ReRunJob:
@@ -963,4 +953,8 @@ class RunInvocation(CheckBoxInvocationMixIn):
         new_matching_job_list = self._get_matching_job_list(
             self.ns, self.state.job_list)
         self._update_desired_job_list(new_matching_job_list)
+        if self._test_plan is not None:
+            job_state = self.state.job_state_map[job.id]
+            job_state.effective_category_id = (
+                self._test_plan.get_effective_category(job))
         self._backtrack_and_run_missing = True
