@@ -29,15 +29,25 @@ from io import BytesIO
 from unittest import TestCase
 
 from plainbox.impl.exporter.text import TextSessionStateExporter
+from plainbox.vendor import mock
 
 
 class TextSessionStateExporterTests(TestCase):
 
     def test_default_dump(self):
-        exporter = TextSessionStateExporter()
+        exporter = TextSessionStateExporter(color=False)
         # Text exporter expects this data format
-        data = {'result_map': {'job_name': {'outcome': 'fail'}}}
+        result = mock.Mock(result='fail', is_hollow=False)
+        result.tr_outcome.return_value = 'fail'
+        job = mock.Mock(id='job_id')
+        job.tr_summary.return_value = 'job name'
+        data = mock.Mock(
+            run_list=[job],
+            job_state_map={
+                job.id: mock.Mock(result=result, job=job)
+            }
+        )
         stream = BytesIO()
         exporter.dump(data, stream)
-        expected_bytes = "job_name: fail\n".encode('UTF-8')
+        expected_bytes = '     fail      : job name\n'.encode('UTF-8')
         self.assertEqual(stream.getvalue(), expected_bytes)
