@@ -5,9 +5,9 @@
 #   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
 #
 # Checkbox is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# it under the terms of the GNU General Public License version 3,
+# as published by the Free Software Foundation.
+
 #
 # Checkbox is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,8 +25,7 @@
 
     THIS MODULE DOES NOT HAVE STABLE PUBLIC API
 """
-
-
+from plainbox.impl.commands.inv_run import Colorizer
 from plainbox.impl.exporter import SessionStateExporterBase
 
 
@@ -35,7 +34,19 @@ class TextSessionStateExporter(SessionStateExporterBase):
     Human-readable session state exporter.
     """
 
-    def dump(self, data, stream):
-        for job_name, job_data in sorted(data['result_map'].items()):
-            stream.write("{}: {}\n".format(
-                job_name, job_data['outcome']).encode('UTF-8'))
+    def __init__(self, option_list=None, color=None):
+        super().__init__(option_list)
+        self.C = Colorizer(color)
+
+    def get_session_data_subset(self, session):
+        return session
+
+    def dump(self, session, stream):
+        for job in session.run_list:
+            state = session.job_state_map[job.id]
+            if state.result.is_hollow:
+                continue
+            stream.write(
+                "{:^15}: {}\n".format(
+                    self.C.result(state.result), state.job.tr_summary(),
+                ).encode("UTF-8"))
