@@ -35,6 +35,7 @@ import inspect
 
 from plainbox.abc import IJobResult
 from plainbox.i18n import gettext as _
+from plainbox.i18n import pgettext as C_
 from plainbox.impl.signal import Signal
 
 logger = logging.getLogger("plainbox.result")
@@ -53,6 +54,28 @@ logger = logging.getLogger("plainbox.result")
 IOLogRecord = namedtuple("IOLogRecord", "delay stream_name data".split())
 
 
+def tr_outcome(outcome):
+    """
+    Get the translated value of ``OUTCOME_`` constant
+    """
+    return {
+        IJobResult.OUTCOME_NONE: C_(
+            "textual outcome", "job didn't run"),
+        IJobResult.OUTCOME_PASS: C_(
+            "textual outcome", "job passed"),
+        IJobResult.OUTCOME_FAIL: C_(
+            "textual outcome", "job failed"),
+        IJobResult.OUTCOME_SKIP: C_(
+            "textual outcome", "job skipped"),
+        IJobResult.OUTCOME_NOT_SUPPORTED: C_(
+            "textual outcome", "job cannot be started"),
+        IJobResult.OUTCOME_NOT_IMPLEMENTED: C_(
+            "textual outcome", "job is not implemented"),
+        IJobResult.OUTCOME_UNDECIDED: C_(
+            "textual outcome", "job needs verification")
+    }[outcome]
+
+
 class _JobResultBase(IJobResult):
     """
     Base class for :`IJobResult` implementations.
@@ -114,14 +137,23 @@ class _JobResultBase(IJobResult):
             self._data['outcome'] = new
             self.on_outcome_changed(old, new)
 
+    def tr_outcome(self):
+        """
+        Get the translated value of the outcome
+        """
+        return tr_outcome(self.outcome)
+
     @property
     def execution_duration(self):
         """
-        The amount of time in seconds it took to run this
-        jobs command.
+        The amount of time in seconds it took to run this job.
         """
         return self._data.get('execution_duration')
 
+    @execution_duration.setter
+    def execution_duration(self, elapsed):
+        self._data['execution_duration'] = elapsed
+
     @property
     def comments(self):
         """
@@ -136,6 +168,23 @@ class _JobResultBase(IJobResult):
             self._data['comments'] = new
             self.on_comments_changed(old, new)
 
+    def append_comments(self, comments):
+        """
+        Append new comments to the result
+
+        :param comments:
+            The comments to append
+
+        This method simplifies appending comments to an existing result.
+        Since initally there are no comments (None is returned) this makes
+        all the code that has to halde updates a little tedious. Using this
+        method, in contrast, is always easy.
+        """
+        if self.comments is None:
+            self.comments = comments
+        else:
+            self.comments = self.comments + '\n' + comments
+
     @Signal.define
     def on_comments_changed(self, old, new):
         """
@@ -153,6 +202,20 @@ class _JobResultBase(IJobResult):
     def io_log(self):
         return tuple(self.get_io_log())
 
+    @property
+    def is_hollow(self):
+        """
+        flag that indicates if the result is hollow
+
+        Hollow results may have been created but hold no data at all.
+        Hollow results are also tentatively deprecated, once we have some
+        time to re-factor SessionState and specifically the job_state_map
+        code we will remove the need to have hollow results.
+
+        Hollow results are not saved, beginning with
+        :class:`plainbox.impl.session.suspend.SessionSuspendHelper4`.
+        """
+        return not bool(self._data)
 
 class MemoryJobResult(_JobResultBase):
     """
