@@ -49,6 +49,8 @@ import binascii
 import gzip
 import json
 import logging
+import os
+import re
 
 from plainbox.abc import IJobResult
 from plainbox.i18n import gettext as _
@@ -94,6 +96,14 @@ class IncompatibleJobError(SessionResumeError):
     """
 
 
+class BrokenReferenceToExternalFile(SessionResumeError):
+    """
+    Exception raised when :class:`SessionResumeHelper` detects that a file
+    needed by the session to resume is not present. This is typically used to
+    signal inaccessible log files.
+    """
+
+
 class EnvelopeUnpackMixIn:
     """
     A mix-in class capable of unpacking the envelope of the session storage
@@ -176,11 +186,14 @@ class SessionResumeHelper(EnvelopeUnpackMixIn):
     appropriate, format specific, resume class.
     """
 
-    def __init__(self, job_list):
+    def __init__(self, job_list, flags=None, location=None):
         """
         Initialize the helper with a list of known jobs.
         """
         self.job_list = job_list
+        logger.debug("Session Resume Helper started with jobs: %r", job_list)
+        self.flags = flags
+        self.location = location
 
     def resume(self, data, early_cb=None):
         """
@@ -230,20 +243,21 @@ class SessionResumeHelper(EnvelopeUnpackMixIn):
         _validate(json_repr, value_type=dict)
         version = _validate(json_repr, key="version", choice=[1])
         if version == 1:
-            return SessionResumeHelper1(
-                self.job_list).resume_json(json_repr, early_cb)
+            helper = SessionResumeHelper1(
+                self.job_list, self.flags, self.location)
         elif version == 2:
-            return SessionResumeHelper2(
-                self.job_list).resume_json(json_repr, early_cb)
+            helper = SessionResumeHelper2(
+                self.job_list, self.flags, self.location)
         elif version == 3:
-            return SessionResumeHelper3(
-                self.job_list).resume_json(json_repr, early_cb)
+            helper = SessionResumeHelper3(
+                self.job_list, self.flags, self.location)
         elif version == 4:
-            return SessionResumeHelper4(
-                self.job_list).resume_json(json_repr, early_cb)
+            helper = SessionResumeHelper4(
+                self.job_list, self.flags, self.location)
         else:
             raise IncompatibleSessionError(
                 _("Unsupported version {}").format(version))
+        return helper.resume_json(json_repr, early_cb)
 
 
 class ResumeDiscardQualifier(SimpleQualifier):
@@ -427,11 +441,42 @@ class SessionResumeHelper1(MetaDataHelper1MixIn):
     failure modes are possible. Those are documented in :meth:`resume()`
     """
 
-    def __init__(self, job_list):
+    # Flag controling reference checks from within the session file to external
+    # files. If enabled such checks are performed and can cause additional
+    # exceptions to be raised. Currently this only affects the representation
+    # of the DiskJobResult instances.
+    FLAG_FILE_REFERENCE_CHECKS_S = 'file-reference-checks'
+    FLAG_FILE_REFERENCE_CHECKS_F = 0x01
+    # Flag controlling rewriting of log file pathnames. It depends on the
+    # location to be non-None and then rewrites pathnames of all them missing
+    # log files to be relative to the session storage location. It effectively
+    # depends on FLAG_FILE_REFERENCe_CHECKS_F being set at the same time,
+    # otherwise it is ignored.
+    FLAG_REWRITE_LOG_PATHNAMES_S = 'rewrite-log-pathnames'
+    FLAG_REWRITE_LOG_PATHNAMES_F = 0x02
+    # Flag controlling integrity checks between jobs present at resume time and
+    # jobs present at suspend time. Since providers cannot be serialized (nor
+    # should they) this integrity check prevents anyone from resuming a session
+    # if job definitions have changed. Using this flag effectively disables
+    # that check.
+    FLAG_IGNORE_JOB_CHECKSUMS_S = 'ignore-job-checksums'
+    FLAG_IGNORE_JOB_CHECKSUMS_F = 0x04
+
+    def __init__(self, job_list, flags=None, location=None):
         """
         Initialize the helper with a list of known jobs.
         """
         self.job_list = job_list
+        self.flags = 0
+        self.location = location
+        # Convert flag string constants into numeric flags
+        if flags is not None:
+            if self.FLAG_FILE_REFERENCE_CHECKS_S in flags:
+                self.flags |= self.FLAG_FILE_REFERENCE_CHECKS_F
+            if self.FLAG_REWRITE_LOG_PATHNAMES_S in flags:
+                self.flags |= self.FLAG_REWRITE_LOG_PATHNAMES_F
+            if self.FLAG_IGNORE_JOB_CHECKSUM_S in flags:
+                self.flags |= self.FLAG_IGNORE_JOB_CHECKSUM_F
 
     def resume_json(self, json_repr, early_cb=None):
         """
@@ -525,7 +570,8 @@ class SessionResumeHelper1(MetaDataHelper1MixIn):
                 try:
                     self._process_job(
                         session, jobs_repr, results_repr, job_id)
-                except KeyError:
+                except KeyError as exc:
+                    logger.debug("Seen KeyError for %r", exc)
                     leftover_jobs.append(job_id)
                 else:
                     leftover_shrunk = True
@@ -568,8 +614,11 @@ class SessionResumeHelper1(MetaDataHelper1MixIn):
         job = session.job_state_map[job_id].job
         # Check if job definition has not changed
         if job.checksum != checksum:
-            raise IncompatibleJobError(
-                _("Definition of job {!r} has changed").format(job_id))
+            if self.flags & self.FLAG_IGNORE_JOB_CHECKSUMS_F:
+                logger.warning(_("Ignoring changes to job %r)"), job_id)
+            else:
+                raise IncompatibleJobError(
+                    _("Definition of job {!r} has changed").format(job_id))
         # The result may not be there. This method is called for all the jobs
         # we're supposed to check but not all such jobs need to have results
         if job.id not in results_repr:
@@ -580,7 +629,8 @@ class SessionResumeHelper1(MetaDataHelper1MixIn):
             results_repr, key=job_id, value_type=list, value_none=True)
         for result_repr in result_list_repr:
             _validate(result_repr, value_type=dict)
-            result = self._build_JobResult(result_repr)
+            result = self._build_JobResult(
+                result_repr, self.flags, self.location)
             result_list.append(result)
         # Show the _LAST_ result to the session. Currently we only store one
         # result but showing the most recent (last) result should be good
@@ -653,7 +703,7 @@ class SessionResumeHelper1(MetaDataHelper1MixIn):
             raise
 
     @classmethod
-    def _build_JobResult(cls, result_repr):
+    def _build_JobResult(cls, result_repr, flags=0, location=None):
         """
         Convert the representation of MemoryJobResult or DiskJobResult
         back into an actual instance.
@@ -673,6 +723,18 @@ class SessionResumeHelper1(MetaDataHelper1MixIn):
         if 'io_log_filename' in result_repr:
             io_log_filename = _validate(
                 result_repr, key='io_log_filename', value_type=str)
+            if (flags & cls.FLAG_FILE_REFERENCE_CHECKS_F
+                    and not os.path.isfile(io_log_filename)
+                    and cls.FLAG_REWRITE_LOG_PATHNAMES in flags):
+                io_log_filename2 = cls._rewrite_pathname(io_log_filename,
+                                                         location)
+                logger.warning(_("Rewrote file name from %r to %r"),
+                               io_log_filename, io_log_filename2)
+                io_log_filename = io_log_filename2
+            if (flags & cls.FLAG_FILE_REFERENCE_CHECKS_F
+                    and not os.path.isfile(io_log_filename)):
+                raise BrokenReferenceToExternalFile(
+                    _("cannot access file: {!r}").format(io_log_filename))
             return DiskJobResult({
                 'outcome': outcome,
                 'comments': comments,
@@ -693,6 +755,11 @@ class SessionResumeHelper1(MetaDataHelper1MixIn):
                 'return_code': return_code
             })
 
+    @classmethod
+    def _rewrite_pathname(cls, pathname, location):
+        return re.sub(
+            '.*\/\.cache\/plainbox\/sessions/[^//]+', location, pathname)
+
     @classmethod
     def _build_IOLogRecord(cls, record_repr):
         """
