@@ -218,7 +218,35 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
 
     @property
     def description(self):
-        return self.get_record_value('description')
+        # since version 0.17 description field should be replaced with
+        # purpose/steps/verification fields. To keep backwards compability
+        # description will be generated by combining new ones if description
+        # field is missing
+        description = self.get_record_value('description')
+        if description is None:
+            # try combining purpose/steps/verification fields
+            description = ""
+            for stage in ['purpose', 'steps', 'verification']:
+                stage_value = self.get_record_value(stage)
+                if stage_value is not None:
+                    description += stage.upper() + ':\n' + stage_value + '\n'
+            description = description.strip()
+            if not description:
+                # combining new description yielded empty string
+                description = None
+        return description
+
+    @property
+    def purpose(self):
+        return self.get_record_value('purpose')
+
+    @property
+    def steps(self):
+        return self.get_record_value('steps')
+
+    @property
+    def verification(self):
+        return self.get_record_value('verification')
 
     @property
     def requires(self):
@@ -308,7 +336,43 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
         """
         Get the translated version of :meth:`description`
         """
-        return self.get_translated_record_value('description')
+        tr_description = self.get_translated_record_value('description')
+        if tr_description is None:
+            # try combining purpose/steps/verification fields
+            tr_stages = {
+                'purpose': _('PURPOSE'),
+                'steps': _('STEPS'),
+                'verification': _('VERIFICATION')
+            }
+            tr_description = ""
+            for stage in ['purpose', 'steps', 'verification']:
+                stage_value = self.get_translated_record_value(stage)
+                if stage_value is not None:
+                    tr_description += (tr_stages[stage] + ':\n'
+                                       + stage_value + '\n')
+            tr_description = tr_description.strip()
+            if not tr_description:
+                # combining new description yielded empty string
+                tr_description = None
+        return tr_description
+
+    def tr_purpose(self):
+        """
+        Get the translated version of :meth:`purpose`
+        """
+        return self.get_translated_record_value('purpose')
+
+    def tr_steps(self):
+        """
+        Get the translated version of :meth:`steps`
+        """
+        return self.get_translated_record_value('steps')
+
+    def tr_verification(self):
+        """
+        Get the translated version of :meth:`verification`
+        """
+        return self.get_translated_record_value('verification')
 
     def get_environ_settings(self):
         """
@@ -501,6 +565,9 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
             imports = 'imports'
             flags = 'flags'
             category_id = 'category_id'
+            purpose = 'purpose'
+            steps = 'steps'
+            verification = 'verification'
 
         field_validators = {
             fields.name: [
@@ -580,13 +647,53 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
                 TemplateVariantFieldValidator,
                 # Description is mandatory for manual jobs
                 PresentFieldValidator(
-                    message=_("manual jobs must have a description"),
-                    onlyif=lambda unit: unit.plugin == 'manual'),
-                # Description is recommended for all other jobs
+                    message=_("manual jobs must have a description field, or a"
+                              " set of purpose, steps, and verification "
+                              "fields"),
+                    onlyif=lambda unit: unit.plugin == 'manual' and
+                    unit.purpose is None and unit.steps is None and
+                    unit.verification is None
+                    ),
+                # Description or a set of purpose, steps and verification
+                # fields is recommended for all other jobs
                 PresentFieldValidator(
                     severity=Severity.advice,
-                    message=_("all jobs should have a description"),
-                    onlyif=lambda unit: unit.plugin != 'manual')
+                    message=_("all jobs should have a description field, or a "
+                              "set of purpose, steps and verification fields"),
+                    onlyif=lambda unit: unit.plugin != 'manual'
+                    and (unit.purpose is None and unit.steps is None and
+                         unit.verification is None)),
+            ],
+            fields.purpose: [
+                TranslatableFieldValidator,
+                PresentFieldValidator(
+                    severity=Severity.advice,
+                    message=("please use purpose, steps, and verification"
+                             " fields. See http://plainbox.readthedocs.org"
+                             "/en/latest/author/faq.html#faq-2"),
+                    onlyif=lambda unit:
+                    unit.startup_user_interaction_required
+                    and unit.get_record_value('summary') is None),
+            ],
+            fields.steps: [
+                TranslatableFieldValidator,
+                PresentFieldValidator(
+                    severity=Severity.advice,
+                    message=("please use purpose, steps, and verification"
+                             " fields. See http://plainbox.readthedocs.org"
+                             "/en/latest/author/faq.html#faq-2"),
+                    onlyif=lambda unit:
+                    unit.startup_user_interaction_required),
+            ],
+            fields.verification: [
+                TranslatableFieldValidator,
+                PresentFieldValidator(
+                    severity=Severity.advice,
+                    message = ("please use purpose, steps, and verification"
+                               " fields. See http://plainbox.readthedocs.org"
+                               "/en/latest/author/faq.html#faq-2"),
+                    onlyif=lambda unit:
+                    unit.startup_user_interaction_required),
             ],
             fields.user: [
                 UntranslatableFieldValidator,
